-- | Utilities for interactive use of Gulik.
module Gulik.Interactive(interactiveMain) where
-- Import a bunch of things, with care towards name collisions.
import Gulik.Types
import Gulik.Eval
import Gulik.BaseEnv
import Gulik.Parser

doLine :: [Value] -> Env -> IO ()
doLine st env = do s <- getLine
                   let ex = gulikParse "(interactive)" s
                   do case ex of
                        Left err -> do print err
                                       doLine st env
                        Right e -> let res = evalEnv env st e
                                   in case res of
                                        Left e -> do putStrLn ("Error: " ++ (show e))
                                                     doLine st env
                                        Right (nst,nenv) -> do print (reverse nst)
                                                               doLine nst nenv

{- | A primitive line-based read-evaluate-print loop, with stack and
     environment carryover between lines.  If a line triggers a parser
     or evaluator error, the error is printed and the state is rolled
     back to before that line. -}
interactiveMain :: IO ()
interactiveMain = do preludeStr <- readFile "prelude.glk"
                     let preludeParse = gulikParse "prelude" preludeStr
                     case preludeParse of
                       Left err -> print err
                       Right prelude -> let preludeEnv = exec baseEnv [] prelude
                                        in case preludeEnv of
                                             Left e -> putStrLn ("Error loading prelude: " ++ (show e))
                                             Right env -> doLine [] env
