-- | Parser for Gulik.
module Gulik.Parser(gulikParse) where
-- Import a bunch of things, with care towards name collisions.
import Gulik.Types
import Text.ParserCombinators.Parsec hiding(State)
import qualified Text.ParserCombinators.Parsec.Token as PT
import Text.ParserCombinators.Parsec.Language

-- Define the Parsec.Language style we want to use.
style :: LanguageDef st
style = haskellStyle { reservedNames = ["apply", "!"], commentStart = "", commentEnd = "", commentLine = "--" }

-- Automatically extract useful parsers from the style.
lexer :: PT.TokenParser st
lexer          = PT.makeTokenParser style
naturalOrFloat = PT.naturalOrFloat lexer
stringLiteral  = PT.stringLiteral lexer
reserved       = PT.reserved lexer
identifier     = PT.identifier lexer
symbol         = PT.symbol lexer
braces         = PT.braces lexer

-- The expression parser itself:
parseExprs :: Parser [Expr]
parseExprs = many parseExpr

parseExpr :: Parser Expr
parseExpr   = parseNum <|> parseString <|> parseName <|> parseBind <|> parseBody <|> parseApply
parseNum    = do x <- naturalOrFloat
                 case x of
                   Left i -> return (ExprInteger i)
                   Right f -> return (ExprDouble f)
parseString = stringLiteral >>= return . ExprString
parseName   = identifier >>= return . ExprName
parseApply  = do (reserved "apply") <|> (reserved "!")
                 return ExprApply
parseBind   = do symbol "/"
                 x <- identifier
                 return (ExprBind x)
parseBody   = do e <- braces parseExprs
                 return (ExprBody e)

-- | Parses a string into a list of Gulik expressions.
gulikParse :: String                   -- ^ Source file name, for error reporting.
           -> String                   -- ^ The text to parse
           -> Either ParseError [Expr] -- ^ A parse error, or the list of expressions.
gulikParse = parse parseExprs
