#!/usr/bin/perl -w

use strict;

my @def_order;
my %defs;
my $cur = "";

my @fields = ('words', 'stack', 'where', 'code');

die unless (scalar(@ARGV) == 1);
my $fn = shift;

open(FILE, "<$fn") or die;
while(<FILE>) {
    chomp;
    if (m/^---- BEGIN ([a-z0-9]+)$/) {
	$cur = $1;
	push @def_order, $cur;
	$defs{$cur}{'words'} = "";
	$defs{$cur}{'stack'} = "";
	$defs{$cur}{'where'} = "";
	$defs{$cur}{'code'} = "";
	next;
    } elsif (m/^---- END$/) {
	$cur = "";
	next;
    }

    next if ($cur eq "");
    if (m/^---- WORDS (.+)$/) {
	$defs{$cur}{'words'} .= $1 . "\n";
    } elsif (m/^---- STACK (.+)$/) {
	$defs{$cur}{'stack'} .= $1 . "\n";
    } elsif (m/^---- WHERE (.+)$/) {
	$defs{$cur}{'where'} .= $1 . "\n";
    } else {
	$defs{$cur}{'code'} .= $_ . "\n";
    }
}
close(FILE);

for my $def (keys(%defs)) {
    for my $field (@fields) {
	$defs{$def}{$field} =~ s/^\n+//m;
	$defs{$def}{$field} =~ s/\n+$//m;
	$defs{$def}{$field} =~ s/&/&amp;/g;
	$defs{$def}{$field} =~ s/</&lt;/g;
	$defs{$def}{$field} =~ s/>/&gt;/g;
    }
}

print <<EOF;
<html>
<head>
<title>Documentation for $fn</title>
</head>
<body>
<h1 align=center>Documentation for <tt>$fn</tt></h1>
<table border=1>
<tr><td><b>Symbol</b></td><td><b>Stack diagram</b></td></tr>
EOF

for my $def (@def_order) {
    my ($words, $stack, $where, $code) = ($defs{$def}{'words'},
        $defs{$def}{'stack'}, $defs{$def}{'where'}, $defs{$def}{'code'});

#    print "<hr>\n";
#    print "<h3>$def</h3>\n";

    print "<tr>\n";
    print "<td>$def</td>\n";

#    if ($words ne "") {
#	print $words . "<p>\n";
#    }

    print "<td>\n";
    if ($stack ne "") {
#	print "<h5>Stack diagram:</h5>\n";
	print "<pre>\n";
	print $stack;
	print "</pre>\n";

	if ($where ne "") {
	    print "<i>&nbsp; &nbsp; &nbsp;where</i>\n";
	    print "<pre>\n";
	    print $where;
	    print "</pre>\n";
	}
    }
    print "</td>\n";
    print "</tr>\n";

#    if ($code ne "") {
#	print "<h5>Implementation:</h5>\n";
#	print "<pre>\n";
#	print $code;
#	print "</pre>\n";
#    }
}

print <<EOF;
</table>
</body>
</html>
EOF
